import java.awt.Color;

import jas.engine.Sim;
import jas.engine.SimModel;
import jas.graphics.plot.TimeSeriesPlotter;
import jas.plot.ColorMap;
import jas.plot.LayerIntGridDrawer;
import jas.plot.LayerObjGridDrawer;
import jas.plot.LayeredSurfaceFrame;
import jas.space.IntGrid;
import jas.statistics.IIntSource;

public class DistrictObserver extends SimModel implements IIntSource{

	public static final int VALUE_FIRMS = 0;
	public static final int VALUE_WORKERS = 1;
	
	private DistrictModel model;
	private LayeredSurfaceFrame world;
	private IntGrid visibility;
	
	private TimeSeriesPlotter plotter;
	
	public DistrictObserver() {
		super();

		model = (DistrictModel) Sim.engine.getModelWithID("DistrictModel");
		if (model == null)
			throw new IllegalArgumentException("Model has not been loaded into memory!");
	}

	@Override
	public void setParameters() {
		// TODO Auto-generated method stub

	}

	public void update() {
	
		visibility.clear();
		for (Firm f : model.firms)
			f.paintDomain(visibility, 1);
		for (Worker w : model.workers)
			w.paintDomain(visibility, 2);
		
	}
	
	@Override
	public void buildModel() {
		visibility = new IntGrid(model.worldXSize, model.worldYSize);
		
		ColorMap cRange = new ColorMap(2);
	    cRange.addColor(0, Color.black);
	    cRange.addColor(1, Color.yellow);
	    cRange.addColor(2, Color.gray);

	    LayerIntGridDrawer vis = new LayerIntGridDrawer("Visibility", visibility, cRange);

		LayerObjGridDrawer firms = new LayerObjGridDrawer("Firms layer", model.firmSpace, Color.green);
		LayerObjGridDrawer workers = new LayerObjGridDrawer("Workers layer", model.workerSpace, Color.blue);
		world = new LayeredSurfaceFrame(model.worldXSize, model.worldYSize, 4);
		world.addLayer(vis);
		world.addLayer(firms);
		world.addLayer(workers);		
		addSimWindow(world);
		
		plotter = new TimeSeriesPlotter("Prices");
		plotter.addSeries("Firms", this, VALUE_FIRMS);
		plotter.addSeries("Workers", this, VALUE_WORKERS);
		addSimWindow(plotter);
		
		eventList.scheduleSimple(0, 1, this, "update");
		eventList.scheduleSimple(0, 1, world, Sim.EVENT_UPDATE);
		eventList.scheduleSimple(0, 1, plotter, Sim.EVENT_UPDATE);
	}

	public int getIntValue(int id) {
		switch (id) {
			case VALUE_FIRMS:
				return model.firms.size();
			case VALUE_WORKERS:
				return model.workers.size();
			default:
				throw new IllegalArgumentException("The value id " + id + " is not recognized.");
		}
	}

}
