

import jas.engine.Sim;
import jas.space.IntGrid;
import jas.space.ObjGrid;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

public class DomainPlayer {

	public static final int MAX_EXTENSION = 200;
	
	private ObjGrid space;
	private Set[][] domain;
	private int x = -1, y = -1;
	private int extension = 0;
	private boolean dead = false;
	
	public DomainPlayer(ObjGrid space, Set[][] domain) {
		this.space = space;
		this.domain = domain;
	}

	public void setAt(int x, int y)
	{
		if (x < 0 || x >= space.getXSize())
			throw new ArrayIndexOutOfBoundsException("The x coodinate " + x + " exceeds the space width " + space.getXSize());
		if (y < 0 || y >= space.getYSize())
			throw new ArrayIndexOutOfBoundsException("The y coodinate " + y + " exceeds the space height " + space.getYSize());
		
		if (x != -1)
		{
			removeFromDomain();
			space.set(x, y, null);
		}	
		
		this.x = x;
		this.y = y;
		space.set(x, y, this);
		
		setDomain();
	}
	
	public void setAtRandom()
	{
		int xCandidate, yCandidate;
		do
		{
			xCandidate = Sim.getRnd().getIntFromTo(0, space.getXSize() - 1);
			yCandidate = Sim.getRnd().getIntFromTo(0, space.getYSize() - 1);	
		} while (space.get(xCandidate, yCandidate) != null);
		
		if (x != -1)
		{
			removeFromDomain();
			space.set(x, y, null);
		}	
		
		x = xCandidate;
		y = yCandidate;
		space.set(x, y, this);
		
		setDomain();
	}
	
	public void setNear(DomainPlayer player)
	{
		int xCandidate = space.torusX(player.getX());
		int yCandidate = space.torusY(player.getY());

		int select = Sim.getRnd().getIntFromTo(0, 3);
		switch (select)
		{
			case 0:
				xCandidate = space.torusX(xCandidate - 1);
				break;
			case 1:
				xCandidate = space.torusX(xCandidate + 1);
				break;
			case 2:
				yCandidate = space.torusY(yCandidate - 1);
				break;
			case 3:
				yCandidate = space.torusY(yCandidate + 1);
				break;
				
		}
		
		if (space.get(xCandidate, yCandidate) != null)
			return;
				
		if (x != -1)
		{
			removeFromDomain();
			space.set(x, y, null);
		}	
		
		x = xCandidate;
		y = yCandidate;
		space.set(x, y, this);
		
		setDomain();
	}

	public Set[][] getDomain() {
		return domain;
	}

	public ObjGrid getSpace() {
		return space;
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}
	
	public int getExtension() {
		return extension;
	}

	public void setExtension(int extension) {
		removeFromDomain();
		this.extension = extension;		
		setDomain();
	}

	public void increaseExtension() {
		if (extension + 1 < MAX_EXTENSION)
			setExtension(extension + 1);
	}
	
	@SuppressWarnings("unchecked")
	private void setDomain() {
		int vx = 1;
		int vy = 0;
		int xd = 0;
		int yd = 0;
		int tvx;    // transit vx;
		int tvy;    // transit vy;
		int step = 1;
		int t = 0;
		int m = 0;
		   
		final int xdim = space.getXSize();
		final int ydim = space.getYSize();
		
		for (int e = 0; e < extension; e++) {
		      // determine segment
			final int xx = (x + xd + xdim) % xdim;
			final int yy = (y + yd + ydim) % ydim;
			domain[xx][yy].add(this);

		    xd += vx;
			yd += vy;
			m++;
			if (m == step) {
				tvx = vy;
				tvy = -vx;
				vx = tvx;
				vy = tvy;
				m = 0;
				t++;
				if (t == 2) {
					t = 0;
					step++;
				}
			};
		}
		
// for (int xx = x - extension; xx < x + extension; xx++ )
//			for (int yy = y - extension; yy < y + extension; yy++ )
//				domain[space.torusX(xx)][space.torusY(yy)].add(this);
	}
	
	private void removeFromDomain() {
		for (int xx = 0; xx < space.getXSize(); xx++ )
			for (int yy = 0; yy < space.getYSize(); yy++ )
				domain[xx][yy].remove(this);
	}
	
	@SuppressWarnings("unchecked")
	public Set getPlayersInMyDomain() {
		Set fullSet = new HashSet();
		
		for (int xx = 0; xx < space.getXSize(); xx++ )
			for (int yy = 0; yy < space.getYSize(); yy++ )
			{
				final Set set = domain[xx][yy]; 
				if (set.contains(this) && set.size() > 1)
					fullSet.addAll(set);
			}
		
		fullSet.remove(this);
		return fullSet;
	}
	
	@SuppressWarnings("unchecked")
	public Set getPlayersInMyDomain(Class classType) {
		Set fullSet = new HashSet();
		
		for (int xx = 0; xx < space.getXSize(); xx++ )
			for (int yy = 0; yy < space.getYSize(); yy++ )
			{
				final Set set = domain[xx][yy]; 
				if (set.contains(this) && set.size() > 1)
				{
					Iterator it = set.iterator();
					while (it.hasNext())
					{
						final Object itm = it.next();
						if (itm.getClass().equals(classType))
							fullSet.add(itm);
					}
				}
			}
		
		fullSet.remove(this);
		return fullSet;
	}
	
	public void paintDomain(IntGrid grid, int colorId)
	{
		if (dead)
			return;
		
		for (int xx = x - extension; xx < x + extension; xx++ )
			for (int yy = y - extension; yy < y + extension; yy++ )
			{
				final int rx = space.torusX(xx);
				final int ry = space.torusY(yy);
				if (domain[rx][ry].contains(this))
					grid.set( rx, ry, colorId);	
			}
				
	}
	
	public boolean isDead() {
		return dead;
	}

	public void delete() {
		removeFromDomain();
		if (x != -1 && y != -1)
			space.set(x, y, null);
		dead = true;
	}
}
