
import jas.engine.Sim;

import java.util.LinkedList;

public class Order {

	public static final int MAX_ORDER_LENGTH = 5;
	public static final int ALPHABET = 4;
	public static final int ORDER_REPETITIONS = 2;

	private long name; // order name
	private int[] specifications;
	private Firm[] processing;
	private long issuedAt; // time when was issued
	private int currentStage = 0; 

	public Order(long name, int numberOfSkills) {
		LinkedList<Integer> specs = new LinkedList<Integer>();
		
		this.name = name;

		int length = Sim.getRnd().getIntFromTo(1, MAX_ORDER_LENGTH);
		// compute repeated steps
		for (int i = 0; i < length; i++) {
			specs.add(Sim.getRnd().getIntFromTo(0, numberOfSkills - 1));
			// decide for repetition
			int rep = Sim.getRnd().getIntFromTo(0, ORDER_REPETITIONS);

			for (int j = 0; j < rep; j++) 
				specs.add(Sim.getRnd().getIntFromTo(0, numberOfSkills - 1));
		}
		
		specifications = new int[specs.size()];
		for (int i = 0; i < specifications.length; i++)
			specifications[i] = specs.get(i);
		
		processing = new Firm[specifications.length];
	}

	public Order(long name, int numberOfSkills, int length) {
		LinkedList<Integer> specs = new LinkedList<Integer>();
		this.name = name;

		//Forse non era necessaria, ma per compatibilit l'ho messa
		if (length >= MAX_ORDER_LENGTH * ORDER_REPETITIONS)
			length = MAX_ORDER_LENGTH * ORDER_REPETITIONS;
		
		// compute repeated steps
		for (int i = 0; i < length; i++) {
			specs.add(Sim.getRnd().getIntFromTo(0, numberOfSkills - 1));
		}
		
		specifications = new int[specs.size()];
		for (int i = 0; i < specifications.length; i++)
			specifications[i] = specs.get(i);
		
		processing = new Firm[specifications.length];
	}

	public Firm getAssignedTo() {
		return processing[currentStage];
	}

	public void setAssignedTo(Firm assignedTo) {
		processing[currentStage] = assignedTo;
	}

	public long getIssuedAt() {
		return issuedAt;
	}

	public void setIssuedAt(int issuedAt) {
		this.issuedAt = issuedAt;
	}

	public int getLenght() {
		return specifications.length;
	}

	public int incrementStage() {
		return currentStage++;
	}
	
	public long getName() {
		return name;
	}

	public int getCurrentSpecification() {
		return specifications[currentStage];
	}

	public int getSpecification(int stage) {
		return specifications[stage];
	}

	public int getCurrentStage() {
		return currentStage;
	}

	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(name);
		buffer.append(": ");
		for (int i = 0; i < specifications.length; i++)
			buffer.append(specifications[i]).append(", ");
			
		buffer.append(" Stage: " + currentStage);
		
		return buffer.toString();
	}
}
